# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import transfer_create_virtual_account_transfer_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.transfer_create_virtual_account_transfer_response import TransferCreateVirtualAccountTransferResponse

__all__ = ["TransferResource", "AsyncTransferResource"]


class TransferResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TransferResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return TransferResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TransferResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return TransferResourceWithStreamingResponse(self)

    def create_virtual_account_transfer(
        self,
        *,
        amount_cents: float,
        destination: str,
        source: str,
        x_idempotency_key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransferCreateVirtualAccountTransferResponse:
        """
        Transfer money between Virtual Accounts or fund a Virtual Account from a Primary
        Account

        Args:
          amount_cents: The amount of money to send in cents.

          destination: The ID of the virtual account to transfer money to.

          source: The ID of the virtual account to transfer money from. Can also be the virtual
              account linked to a primary Slash account to fund a new virtual account (Virtual
              account with the name 'Primary account').

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"X-Idempotency-Key": x_idempotency_key, **(extra_headers or {})}
        return self._post(
            "/transfer/virtual-account",
            body=maybe_transform(
                {
                    "amount_cents": amount_cents,
                    "destination": destination,
                    "source": source,
                },
                transfer_create_virtual_account_transfer_params.TransferCreateVirtualAccountTransferParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransferCreateVirtualAccountTransferResponse,
        )


class AsyncTransferResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTransferResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTransferResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTransferResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncTransferResourceWithStreamingResponse(self)

    async def create_virtual_account_transfer(
        self,
        *,
        amount_cents: float,
        destination: str,
        source: str,
        x_idempotency_key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransferCreateVirtualAccountTransferResponse:
        """
        Transfer money between Virtual Accounts or fund a Virtual Account from a Primary
        Account

        Args:
          amount_cents: The amount of money to send in cents.

          destination: The ID of the virtual account to transfer money to.

          source: The ID of the virtual account to transfer money from. Can also be the virtual
              account linked to a primary Slash account to fund a new virtual account (Virtual
              account with the name 'Primary account').

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"X-Idempotency-Key": x_idempotency_key, **(extra_headers or {})}
        return await self._post(
            "/transfer/virtual-account",
            body=await async_maybe_transform(
                {
                    "amount_cents": amount_cents,
                    "destination": destination,
                    "source": source,
                },
                transfer_create_virtual_account_transfer_params.TransferCreateVirtualAccountTransferParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransferCreateVirtualAccountTransferResponse,
        )


class TransferResourceWithRawResponse:
    def __init__(self, transfer: TransferResource) -> None:
        self._transfer = transfer

        self.create_virtual_account_transfer = to_raw_response_wrapper(
            transfer.create_virtual_account_transfer,
        )


class AsyncTransferResourceWithRawResponse:
    def __init__(self, transfer: AsyncTransferResource) -> None:
        self._transfer = transfer

        self.create_virtual_account_transfer = async_to_raw_response_wrapper(
            transfer.create_virtual_account_transfer,
        )


class TransferResourceWithStreamingResponse:
    def __init__(self, transfer: TransferResource) -> None:
        self._transfer = transfer

        self.create_virtual_account_transfer = to_streamed_response_wrapper(
            transfer.create_virtual_account_transfer,
        )


class AsyncTransferResourceWithStreamingResponse:
    def __init__(self, transfer: AsyncTransferResource) -> None:
        self._transfer = transfer

        self.create_virtual_account_transfer = async_to_streamed_response_wrapper(
            transfer.create_virtual_account_transfer,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .account import Account
from .._models import BaseModel
from .pagination_response import PaginationResponse

__all__ = ["AccountListResponse"]


class AccountListResponse(BaseModel):
    items: List[Account]

    metadata: PaginationResponse
    """Response sent when requesting a list of data"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .card_group import (
    CardGroupResource,
    AsyncCardGroupResource,
    CardGroupResourceWithRawResponse,
    AsyncCardGroupResourceWithRawResponse,
    CardGroupResourceWithStreamingResponse,
    AsyncCardGroupResourceWithStreamingResponse,
)
from .spending_constraint import (
    SpendingConstraintResource,
    AsyncSpendingConstraintResource,
    SpendingConstraintResourceWithRawResponse,
    AsyncSpendingConstraintResourceWithRawResponse,
    SpendingConstraintResourceWithStreamingResponse,
    AsyncSpendingConstraintResourceWithStreamingResponse,
)

__all__ = [
    "SpendingConstraintResource",
    "AsyncSpendingConstraintResource",
    "SpendingConstraintResourceWithRawResponse",
    "AsyncSpendingConstraintResourceWithRawResponse",
    "SpendingConstraintResourceWithStreamingResponse",
    "AsyncSpendingConstraintResourceWithStreamingResponse",
    "CardGroupResource",
    "AsyncCardGroupResource",
    "CardGroupResourceWithRawResponse",
    "AsyncCardGroupResourceWithRawResponse",
    "CardGroupResourceWithStreamingResponse",
    "AsyncCardGroupResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
import base64
from typing import Any, Union, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Omit,
    Headers,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from .resources import (
    pay,
    crypto,
    account,
    webhook,
    merchant,
    transfer,
    well_known,
    transaction,
    card_product,
    legal_entity,
    slash_handle,
    virtual_account,
    developer_account,
    merchant_category,
    developer_application,
)
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)
from .resources.fdx import fdx
from .resources.card import card
from .resources.oauth2 import oauth2
from .resources.card_group import card_group

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "SlashSDK",
    "AsyncSlashSDK",
    "Client",
    "AsyncClient",
]


class SlashSDK(SyncAPIClient):
    legal_entity: legal_entity.LegalEntityResource
    account: account.AccountResource
    virtual_account: virtual_account.VirtualAccountResource
    transaction: transaction.TransactionResource
    transfer: transfer.TransferResource
    card: card.CardResource
    card_group: card_group.CardGroupResource
    card_product: card_product.CardProductResource
    slash_handle: slash_handle.SlashHandleResource
    pay: pay.PayResource
    webhook: webhook.WebhookResource
    merchant: merchant.MerchantResource
    merchant_category: merchant_category.MerchantCategoryResource
    developer_account: developer_account.DeveloperAccountResource
    developer_application: developer_application.DeveloperApplicationResource
    well_known: well_known.WellKnownResource
    oauth2: oauth2.Oauth2Resource
    fdx: fdx.FdxResource
    crypto: crypto.CryptoResource
    with_raw_response: SlashSDKWithRawResponse
    with_streaming_response: SlashSDKWithStreamedResponse

    # client options
    api_key: str | None
    bearer_token: str | None
    username: str | None
    password: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        bearer_token: str | None = None,
        username: str | None = None,
        password: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous SlashSDK client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `SLASH_SDK_API_KEY`
        - `bearer_token` from `SLASH_SDK_BEARER_TOKEN`
        - `username` from `SLASH_SDK_USERNAME`
        - `password` from `SLASH_SDK_PASSWORD`
        """
        if api_key is None:
            api_key = os.environ.get("SLASH_SDK_API_KEY")
        self.api_key = api_key

        if bearer_token is None:
            bearer_token = os.environ.get("SLASH_SDK_BEARER_TOKEN")
        self.bearer_token = bearer_token

        if username is None:
            username = os.environ.get("SLASH_SDK_USERNAME")
        self.username = username

        if password is None:
            password = os.environ.get("SLASH_SDK_PASSWORD")
        self.password = password

        if base_url is None:
            base_url = os.environ.get("SLASH_SDK_BASE_URL")
        if base_url is None:
            base_url = f"https://api.joinslash.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.legal_entity = legal_entity.LegalEntityResource(self)
        self.account = account.AccountResource(self)
        self.virtual_account = virtual_account.VirtualAccountResource(self)
        self.transaction = transaction.TransactionResource(self)
        self.transfer = transfer.TransferResource(self)
        self.card = card.CardResource(self)
        self.card_group = card_group.CardGroupResource(self)
        self.card_product = card_product.CardProductResource(self)
        self.slash_handle = slash_handle.SlashHandleResource(self)
        self.pay = pay.PayResource(self)
        self.webhook = webhook.WebhookResource(self)
        self.merchant = merchant.MerchantResource(self)
        self.merchant_category = merchant_category.MerchantCategoryResource(self)
        self.developer_account = developer_account.DeveloperAccountResource(self)
        self.developer_application = developer_application.DeveloperApplicationResource(self)
        self.well_known = well_known.WellKnownResource(self)
        self.oauth2 = oauth2.Oauth2Resource(self)
        self.fdx = fdx.FdxResource(self)
        self.crypto = crypto.CryptoResource(self)
        self.with_raw_response = SlashSDKWithRawResponse(self)
        self.with_streaming_response = SlashSDKWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        return {**self._api_key, **self._bearer, **self._developer_application}

    @property
    def _api_key(self) -> dict[str, str]:
        api_key = self.api_key
        if api_key is None:
            return {}
        return {"X-API-Key": api_key}

    @property
    def _bearer(self) -> dict[str, str]:
        bearer_token = self.bearer_token
        if bearer_token is None:
            return {}
        return {"Authorization": f"Bearer {bearer_token}"}

    @property
    def _developer_application(self) -> dict[str, str]:
        if self.username is None:
            return {}
        if self.password is None:
            return {}
        credentials = f"{self.username}:{self.password}".encode("ascii")
        header = f"Basic {base64.b64encode(credentials).decode('ascii')}"
        return {"Authorization": header}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    @override
    def _validate_headers(self, headers: Headers, custom_headers: Headers) -> None:
        if self.api_key and headers.get("X-API-Key"):
            return
        if isinstance(custom_headers.get("X-API-Key"), Omit):
            return

        if self.bearer_token and headers.get("Authorization"):
            return
        if isinstance(custom_headers.get("Authorization"), Omit):
            return

        if self.username and self.password and headers.get("Authorization"):
            return
        if isinstance(custom_headers.get("Authorization"), Omit):
            return

        raise TypeError(
            '"Could not resolve authentication method. Expected one of api_key, bearer_token, username or password to be set. Or for one of the `X-API-Key`, `Authorization` or `Authorization` headers to be explicitly omitted"'
        )

    def copy(
        self,
        *,
        api_key: str | None = None,
        bearer_token: str | None = None,
        username: str | None = None,
        password: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            bearer_token=bearer_token or self.bearer_token,
            username=username or self.username,
            password=password or self.password,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncSlashSDK(AsyncAPIClient):
    legal_entity: legal_entity.AsyncLegalEntityResource
    account: account.AsyncAccountResource
    virtual_account: virtual_account.AsyncVirtualAccountResource
    transaction: transaction.AsyncTransactionResource
    transfer: transfer.AsyncTransferResource
    card: card.AsyncCardResource
    card_group: card_group.AsyncCardGroupResource
    card_product: card_product.AsyncCardProductResource
    slash_handle: slash_handle.AsyncSlashHandleResource
    pay: pay.AsyncPayResource
    webhook: webhook.AsyncWebhookResource
    merchant: merchant.AsyncMerchantResource
    merchant_category: merchant_category.AsyncMerchantCategoryResource
    developer_account: developer_account.AsyncDeveloperAccountResource
    developer_application: developer_application.AsyncDeveloperApplicationResource
    well_known: well_known.AsyncWellKnownResource
    oauth2: oauth2.AsyncOauth2Resource
    fdx: fdx.AsyncFdxResource
    crypto: crypto.AsyncCryptoResource
    with_raw_response: AsyncSlashSDKWithRawResponse
    with_streaming_response: AsyncSlashSDKWithStreamedResponse

    # client options
    api_key: str | None
    bearer_token: str | None
    username: str | None
    password: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        bearer_token: str | None = None,
        username: str | None = None,
        password: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncSlashSDK client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `SLASH_SDK_API_KEY`
        - `bearer_token` from `SLASH_SDK_BEARER_TOKEN`
        - `username` from `SLASH_SDK_USERNAME`
        - `password` from `SLASH_SDK_PASSWORD`
        """
        if api_key is None:
            api_key = os.environ.get("SLASH_SDK_API_KEY")
        self.api_key = api_key

        if bearer_token is None:
            bearer_token = os.environ.get("SLASH_SDK_BEARER_TOKEN")
        self.bearer_token = bearer_token

        if username is None:
            username = os.environ.get("SLASH_SDK_USERNAME")
        self.username = username

        if password is None:
            password = os.environ.get("SLASH_SDK_PASSWORD")
        self.password = password

        if base_url is None:
            base_url = os.environ.get("SLASH_SDK_BASE_URL")
        if base_url is None:
            base_url = f"https://api.joinslash.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.legal_entity = legal_entity.AsyncLegalEntityResource(self)
        self.account = account.AsyncAccountResource(self)
        self.virtual_account = virtual_account.AsyncVirtualAccountResource(self)
        self.transaction = transaction.AsyncTransactionResource(self)
        self.transfer = transfer.AsyncTransferResource(self)
        self.card = card.AsyncCardResource(self)
        self.card_group = card_group.AsyncCardGroupResource(self)
        self.card_product = card_product.AsyncCardProductResource(self)
        self.slash_handle = slash_handle.AsyncSlashHandleResource(self)
        self.pay = pay.AsyncPayResource(self)
        self.webhook = webhook.AsyncWebhookResource(self)
        self.merchant = merchant.AsyncMerchantResource(self)
        self.merchant_category = merchant_category.AsyncMerchantCategoryResource(self)
        self.developer_account = developer_account.AsyncDeveloperAccountResource(self)
        self.developer_application = developer_application.AsyncDeveloperApplicationResource(self)
        self.well_known = well_known.AsyncWellKnownResource(self)
        self.oauth2 = oauth2.AsyncOauth2Resource(self)
        self.fdx = fdx.AsyncFdxResource(self)
        self.crypto = crypto.AsyncCryptoResource(self)
        self.with_raw_response = AsyncSlashSDKWithRawResponse(self)
        self.with_streaming_response = AsyncSlashSDKWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        return {**self._api_key, **self._bearer, **self._developer_application}

    @property
    def _api_key(self) -> dict[str, str]:
        api_key = self.api_key
        if api_key is None:
            return {}
        return {"X-API-Key": api_key}

    @property
    def _bearer(self) -> dict[str, str]:
        bearer_token = self.bearer_token
        if bearer_token is None:
            return {}
        return {"Authorization": f"Bearer {bearer_token}"}

    @property
    def _developer_application(self) -> dict[str, str]:
        if self.username is None:
            return {}
        if self.password is None:
            return {}
        credentials = f"{self.username}:{self.password}".encode("ascii")
        header = f"Basic {base64.b64encode(credentials).decode('ascii')}"
        return {"Authorization": header}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    @override
    def _validate_headers(self, headers: Headers, custom_headers: Headers) -> None:
        if self.api_key and headers.get("X-API-Key"):
            return
        if isinstance(custom_headers.get("X-API-Key"), Omit):
            return

        if self.bearer_token and headers.get("Authorization"):
            return
        if isinstance(custom_headers.get("Authorization"), Omit):
            return

        if self.username and self.password and headers.get("Authorization"):
            return
        if isinstance(custom_headers.get("Authorization"), Omit):
            return

        raise TypeError(
            '"Could not resolve authentication method. Expected one of api_key, bearer_token, username or password to be set. Or for one of the `X-API-Key`, `Authorization` or `Authorization` headers to be explicitly omitted"'
        )

    def copy(
        self,
        *,
        api_key: str | None = None,
        bearer_token: str | None = None,
        username: str | None = None,
        password: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            bearer_token=bearer_token or self.bearer_token,
            username=username or self.username,
            password=password or self.password,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class SlashSDKWithRawResponse:
    def __init__(self, client: SlashSDK) -> None:
        self.legal_entity = legal_entity.LegalEntityResourceWithRawResponse(client.legal_entity)
        self.account = account.AccountResourceWithRawResponse(client.account)
        self.virtual_account = virtual_account.VirtualAccountResourceWithRawResponse(client.virtual_account)
        self.transaction = transaction.TransactionResourceWithRawResponse(client.transaction)
        self.transfer = transfer.TransferResourceWithRawResponse(client.transfer)
        self.card = card.CardResourceWithRawResponse(client.card)
        self.card_group = card_group.CardGroupResourceWithRawResponse(client.card_group)
        self.card_product = card_product.CardProductResourceWithRawResponse(client.card_product)
        self.slash_handle = slash_handle.SlashHandleResourceWithRawResponse(client.slash_handle)
        self.pay = pay.PayResourceWithRawResponse(client.pay)
        self.webhook = webhook.WebhookResourceWithRawResponse(client.webhook)
        self.merchant = merchant.MerchantResourceWithRawResponse(client.merchant)
        self.merchant_category = merchant_category.MerchantCategoryResourceWithRawResponse(client.merchant_category)
        self.developer_account = developer_account.DeveloperAccountResourceWithRawResponse(client.developer_account)
        self.developer_application = developer_application.DeveloperApplicationResourceWithRawResponse(
            client.developer_application
        )
        self.well_known = well_known.WellKnownResourceWithRawResponse(client.well_known)
        self.oauth2 = oauth2.Oauth2ResourceWithRawResponse(client.oauth2)
        self.fdx = fdx.FdxResourceWithRawResponse(client.fdx)
        self.crypto = crypto.CryptoResourceWithRawResponse(client.crypto)


class AsyncSlashSDKWithRawResponse:
    def __init__(self, client: AsyncSlashSDK) -> None:
        self.legal_entity = legal_entity.AsyncLegalEntityResourceWithRawResponse(client.legal_entity)
        self.account = account.AsyncAccountResourceWithRawResponse(client.account)
        self.virtual_account = virtual_account.AsyncVirtualAccountResourceWithRawResponse(client.virtual_account)
        self.transaction = transaction.AsyncTransactionResourceWithRawResponse(client.transaction)
        self.transfer = transfer.AsyncTransferResourceWithRawResponse(client.transfer)
        self.card = card.AsyncCardResourceWithRawResponse(client.card)
        self.card_group = card_group.AsyncCardGroupResourceWithRawResponse(client.card_group)
        self.card_product = card_product.AsyncCardProductResourceWithRawResponse(client.card_product)
        self.slash_handle = slash_handle.AsyncSlashHandleResourceWithRawResponse(client.slash_handle)
        self.pay = pay.AsyncPayResourceWithRawResponse(client.pay)
        self.webhook = webhook.AsyncWebhookResourceWithRawResponse(client.webhook)
        self.merchant = merchant.AsyncMerchantResourceWithRawResponse(client.merchant)
        self.merchant_category = merchant_category.AsyncMerchantCategoryResourceWithRawResponse(
            client.merchant_category
        )
        self.developer_account = developer_account.AsyncDeveloperAccountResourceWithRawResponse(
            client.developer_account
        )
        self.developer_application = developer_application.AsyncDeveloperApplicationResourceWithRawResponse(
            client.developer_application
        )
        self.well_known = well_known.AsyncWellKnownResourceWithRawResponse(client.well_known)
        self.oauth2 = oauth2.AsyncOauth2ResourceWithRawResponse(client.oauth2)
        self.fdx = fdx.AsyncFdxResourceWithRawResponse(client.fdx)
        self.crypto = crypto.AsyncCryptoResourceWithRawResponse(client.crypto)


class SlashSDKWithStreamedResponse:
    def __init__(self, client: SlashSDK) -> None:
        self.legal_entity = legal_entity.LegalEntityResourceWithStreamingResponse(client.legal_entity)
        self.account = account.AccountResourceWithStreamingResponse(client.account)
        self.virtual_account = virtual_account.VirtualAccountResourceWithStreamingResponse(client.virtual_account)
        self.transaction = transaction.TransactionResourceWithStreamingResponse(client.transaction)
        self.transfer = transfer.TransferResourceWithStreamingResponse(client.transfer)
        self.card = card.CardResourceWithStreamingResponse(client.card)
        self.card_group = card_group.CardGroupResourceWithStreamingResponse(client.card_group)
        self.card_product = card_product.CardProductResourceWithStreamingResponse(client.card_product)
        self.slash_handle = slash_handle.SlashHandleResourceWithStreamingResponse(client.slash_handle)
        self.pay = pay.PayResourceWithStreamingResponse(client.pay)
        self.webhook = webhook.WebhookResourceWithStreamingResponse(client.webhook)
        self.merchant = merchant.MerchantResourceWithStreamingResponse(client.merchant)
        self.merchant_category = merchant_category.MerchantCategoryResourceWithStreamingResponse(
            client.merchant_category
        )
        self.developer_account = developer_account.DeveloperAccountResourceWithStreamingResponse(
            client.developer_account
        )
        self.developer_application = developer_application.DeveloperApplicationResourceWithStreamingResponse(
            client.developer_application
        )
        self.well_known = well_known.WellKnownResourceWithStreamingResponse(client.well_known)
        self.oauth2 = oauth2.Oauth2ResourceWithStreamingResponse(client.oauth2)
        self.fdx = fdx.FdxResourceWithStreamingResponse(client.fdx)
        self.crypto = crypto.CryptoResourceWithStreamingResponse(client.crypto)


class AsyncSlashSDKWithStreamedResponse:
    def __init__(self, client: AsyncSlashSDK) -> None:
        self.legal_entity = legal_entity.AsyncLegalEntityResourceWithStreamingResponse(client.legal_entity)
        self.account = account.AsyncAccountResourceWithStreamingResponse(client.account)
        self.virtual_account = virtual_account.AsyncVirtualAccountResourceWithStreamingResponse(client.virtual_account)
        self.transaction = transaction.AsyncTransactionResourceWithStreamingResponse(client.transaction)
        self.transfer = transfer.AsyncTransferResourceWithStreamingResponse(client.transfer)
        self.card = card.AsyncCardResourceWithStreamingResponse(client.card)
        self.card_group = card_group.AsyncCardGroupResourceWithStreamingResponse(client.card_group)
        self.card_product = card_product.AsyncCardProductResourceWithStreamingResponse(client.card_product)
        self.slash_handle = slash_handle.AsyncSlashHandleResourceWithStreamingResponse(client.slash_handle)
        self.pay = pay.AsyncPayResourceWithStreamingResponse(client.pay)
        self.webhook = webhook.AsyncWebhookResourceWithStreamingResponse(client.webhook)
        self.merchant = merchant.AsyncMerchantResourceWithStreamingResponse(client.merchant)
        self.merchant_category = merchant_category.AsyncMerchantCategoryResourceWithStreamingResponse(
            client.merchant_category
        )
        self.developer_account = developer_account.AsyncDeveloperAccountResourceWithStreamingResponse(
            client.developer_account
        )
        self.developer_application = developer_application.AsyncDeveloperApplicationResourceWithStreamingResponse(
            client.developer_application
        )
        self.well_known = well_known.AsyncWellKnownResourceWithStreamingResponse(client.well_known)
        self.oauth2 = oauth2.AsyncOauth2ResourceWithStreamingResponse(client.oauth2)
        self.fdx = fdx.AsyncFdxResourceWithStreamingResponse(client.fdx)
        self.crypto = crypto.AsyncCryptoResourceWithStreamingResponse(client.crypto)


Client = SlashSDK

AsyncClient = AsyncSlashSDK



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .pagination_response import PaginationResponse

__all__ = ["LegalEntityListResponse", "Item"]


class Item(BaseModel):
    id: str

    name: str

    structure: Optional[Literal["person", "llc", "lp", "llp", "gp", "partnership", "ccorp", "scorp"]] = None
    """
    Will be empty in rare cases where the legal entity has not yet been fully
    created.
    """


class LegalEntityListResponse(BaseModel):
    items: List[Item]

    metadata: PaginationResponse
    """Response sent when requesting a list of data"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .accounts import (
    AccountsResource,
    AsyncAccountsResource,
    AccountsResourceWithRawResponse,
    AsyncAccountsResourceWithRawResponse,
    AccountsResourceWithStreamingResponse,
    AsyncAccountsResourceWithStreamingResponse,
)
from .statements import (
    StatementsResource,
    AsyncStatementsResource,
    StatementsResourceWithRawResponse,
    AsyncStatementsResourceWithRawResponse,
    StatementsResourceWithStreamingResponse,
    AsyncStatementsResourceWithStreamingResponse,
)

__all__ = [
    "StatementsResource",
    "AsyncStatementsResource",
    "StatementsResourceWithRawResponse",
    "AsyncStatementsResourceWithRawResponse",
    "StatementsResourceWithStreamingResponse",
    "AsyncStatementsResourceWithStreamingResponse",
    "AccountsResource",
    "AsyncAccountsResource",
    "AccountsResourceWithRawResponse",
    "AsyncAccountsResourceWithRawResponse",
    "AccountsResourceWithStreamingResponse",
    "AsyncAccountsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from slash_sdk import SlashSDK, AsyncSlashSDK
from tests.utils import assert_matches_type
from slash_sdk.types import DeveloperApplicationModel

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDeveloperApplication:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: SlashSDK) -> None:
        developer_application = client.developer_application.retrieve(
            "developerApplicationId",
        )
        assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: SlashSDK) -> None:
        response = client.developer_application.with_raw_response.retrieve(
            "developerApplicationId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        developer_application = response.parse()
        assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: SlashSDK) -> None:
        with client.developer_application.with_streaming_response.retrieve(
            "developerApplicationId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            developer_application = response.parse()
            assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: SlashSDK) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `developer_application_id` but received ''"
        ):
            client.developer_application.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: SlashSDK) -> None:
        developer_application = client.developer_application.update(
            developer_application_id="developerApplicationId",
        )
        assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: SlashSDK) -> None:
        developer_application = client.developer_application.update(
            developer_application_id="developerApplicationId",
            data={
                "allowed_origins": ["string"],
                "description": "description",
                "scopes": ["legal_entity.home.view"],
                "type": "service",
                "logo_url": "logoUrl",
                "privacy_policy_url": "privacyPolicyUrl",
                "redirect_uris": ["string"],
                "terms_of_service_url": "termsOfServiceUrl",
                "test_user_emails": ["string"],
            },
            name="name",
        )
        assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: SlashSDK) -> None:
        response = client.developer_application.with_raw_response.update(
            developer_application_id="developerApplicationId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        developer_application = response.parse()
        assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: SlashSDK) -> None:
        with client.developer_application.with_streaming_response.update(
            developer_application_id="developerApplicationId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            developer_application = response.parse()
            assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update(self, client: SlashSDK) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `developer_application_id` but received ''"
        ):
            client.developer_application.with_raw_response.update(
                developer_application_id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_create_or_regenerate_secret(self, client: SlashSDK) -> None:
        developer_application = client.developer_application.create_or_regenerate_secret(
            "developerApplicationId",
        )
        assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create_or_regenerate_secret(self, client: SlashSDK) -> None:
        response = client.developer_application.with_raw_response.create_or_regenerate_secret(
            "developerApplicationId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        developer_application = response.parse()
        assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create_or_regenerate_secret(self, client: SlashSDK) -> None:
        with client.developer_application.with_streaming_response.create_or_regenerate_secret(
            "developerApplicationId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            developer_application = response.parse()
            assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_create_or_regenerate_secret(self, client: SlashSDK) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `developer_application_id` but received ''"
        ):
            client.developer_application.with_raw_response.create_or_regenerate_secret(
                "",
            )


class TestAsyncDeveloperApplication:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSlashSDK) -> None:
        developer_application = await async_client.developer_application.retrieve(
            "developerApplicationId",
        )
        assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.developer_application.with_raw_response.retrieve(
            "developerApplicationId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        developer_application = await response.parse()
        assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.developer_application.with_streaming_response.retrieve(
            "developerApplicationId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            developer_application = await response.parse()
            assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `developer_application_id` but received ''"
        ):
            await async_client.developer_application.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncSlashSDK) -> None:
        developer_application = await async_client.developer_application.update(
            developer_application_id="developerApplicationId",
        )
        assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        developer_application = await async_client.developer_application.update(
            developer_application_id="developerApplicationId",
            data={
                "allowed_origins": ["string"],
                "description": "description",
                "scopes": ["legal_entity.home.view"],
                "type": "service",
                "logo_url": "logoUrl",
                "privacy_policy_url": "privacyPolicyUrl",
                "redirect_uris": ["string"],
                "terms_of_service_url": "termsOfServiceUrl",
                "test_user_emails": ["string"],
            },
            name="name",
        )
        assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.developer_application.with_raw_response.update(
            developer_application_id="developerApplicationId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        developer_application = await response.parse()
        assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.developer_application.with_streaming_response.update(
            developer_application_id="developerApplicationId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            developer_application = await response.parse()
            assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `developer_application_id` but received ''"
        ):
            await async_client.developer_application.with_raw_response.update(
                developer_application_id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_or_regenerate_secret(self, async_client: AsyncSlashSDK) -> None:
        developer_application = await async_client.developer_application.create_or_regenerate_secret(
            "developerApplicationId",
        )
        assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create_or_regenerate_secret(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.developer_application.with_raw_response.create_or_regenerate_secret(
            "developerApplicationId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        developer_application = await response.parse()
        assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create_or_regenerate_secret(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.developer_application.with_streaming_response.create_or_regenerate_secret(
            "developerApplicationId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            developer_application = await response.parse()
            assert_matches_type(DeveloperApplicationModel, developer_application, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_create_or_regenerate_secret(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `developer_application_id` but received ''"
        ):
            await async_client.developer_application.with_raw_response.create_or_regenerate_secret(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["TransactionListParams"]


class TransactionListParams(TypedDict, total=False):
    account_id: Annotated[str, PropertyInfo(alias="accountId")]
    """Use filter:accountId to filter by account ID"""

    cursor: str
    """A cursor string to fetch the next page of results"""

    filter_account_id: Annotated[str, PropertyInfo(alias="filter:accountId")]
    """Pass in an account ID to filter transactions by account ID.

    This will return all transactions that match the account ID passed in.
    """

    filter_card_id: Annotated[str, PropertyInfo(alias="filter:cardId")]
    """Filter transactions by cardId"""

    filter_detailed_status: Annotated[
        Literal["pending", "canceled", "failed", "settled", "declined", "refund", "reversed", "returned", "dispute"],
        PropertyInfo(alias="filter:detailed_status"),
    ]
    """Filter transactions by detailed status"""

    filter_from_authorized_at: Annotated[str, PropertyInfo(alias="filter:from_authorized_at")]
    """
    Pass in a unix timestamp in milliseconds to filter transactions by authorization
    time. This will return all transactions that are authorized on or after the date
    passed in.
    """

    filter_from_date: Annotated[str, PropertyInfo(alias="filter:from_date")]
    """Pass in a unix timestamp in milliseconds to filter transactions by date.

    This will return all transactions that occurred on or after the date passed in.
    """

    filter_legal_entity_id: Annotated[str, PropertyInfo(alias="filter:legalEntityId")]
    """
    Pass in a legal entity ID to filter transactions by accounts under a specific
    legal entity.
    """

    filter_status: Annotated[Literal["pending", "posted", "failed"], PropertyInfo(alias="filter:status")]
    """Filter transactions by status"""

    filter_to_authorized_at: Annotated[str, PropertyInfo(alias="filter:to_authorized_at")]
    """
    Pass in a unix timestamp in milliseconds to filter transactions by authorization
    time. This will return all transactions that are authorized on or before the
    date passed in.
    """

    filter_to_date: Annotated[str, PropertyInfo(alias="filter:to_date")]
    """Pass in a unix timestamp in milliseconds to filter transactions by date.

    This will return all transactions that occurred on or before the date passed in.
    """

    filter_virtual_account_id: Annotated[str, PropertyInfo(alias="filter:virtualAccountId")]
    """Pass in a virtual account ID to filter transactions by virtual account ID.

    This will return all transactions that match the virtual account ID passed in.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .oauth2 import (
    Oauth2Resource,
    AsyncOauth2Resource,
    Oauth2ResourceWithRawResponse,
    AsyncOauth2ResourceWithRawResponse,
    Oauth2ResourceWithStreamingResponse,
    AsyncOauth2ResourceWithStreamingResponse,
)
from .userinfo import (
    UserinfoResource,
    AsyncUserinfoResource,
    UserinfoResourceWithRawResponse,
    AsyncUserinfoResourceWithRawResponse,
    UserinfoResourceWithStreamingResponse,
    AsyncUserinfoResourceWithStreamingResponse,
)

__all__ = [
    "UserinfoResource",
    "AsyncUserinfoResource",
    "UserinfoResourceWithRawResponse",
    "AsyncUserinfoResourceWithRawResponse",
    "UserinfoResourceWithStreamingResponse",
    "AsyncUserinfoResourceWithStreamingResponse",
    "Oauth2Resource",
    "AsyncOauth2Resource",
    "Oauth2ResourceWithRawResponse",
    "AsyncOauth2ResourceWithRawResponse",
    "Oauth2ResourceWithStreamingResponse",
    "AsyncOauth2ResourceWithStreamingResponse",
]
